@@ -5,7 +5,7 @@
 //!  - communicate and enforce a limit on the amount of elements to store
 //!  - efficiently pop an arbitrary amount of elements from the front (optionally into a slice to
 //!    avoid uneccessary reallocations)
-//!  - access the underlying buffer directly by using either `peek*` methods or (range-)indices
+//!  - access the underlying buffer directly by using (range-)indices
 //!  - dereference the `SliceQueue<T>` by propagating the `deref()`-call to the underlying `Vec<T>`
 
 use std::{
@@ -229,44 +229,6 @@ impl<T> SliceQueue<T> {
 	}
 	
 	
-	/// Returns a reference to the first element
-	///
-	/// Returns either _`Some(element_reference)`_ if there is an element to reference or _`None`_
-	/// otherwise
-	pub fn peek(&self) -> Option<&T> {
-		self.backing.first()
-	}
-	/// Returns a mutable reference to the first element
-	///
-	/// Returns either _`Some(element_reference)`_ if there is an element to reference or _`None`_
-	/// otherwise
-	pub fn peek_mut(&mut self) -> Option<&mut T> {
-		self.backing.first_mut()
-	}
-	/// Returns a reference to the first `n` elements
-	///
-	/// Parameters:
-	///  - `n`: The amount of elements to reference
-	///
-	/// Returns either _`Some(element_references)`_ if there are enough elements to reference or
-	/// _`None`_ otherwise
-	pub fn peek_n(&self, n: usize) -> Option<&[T]> {
-		if self.len() < n { None }
-			else { Some(&self.backing[..n]) }
-	}
-	/// Returns a mutable reference to the first `n` elements
-	///
-	/// Parameters:
-	///  - `n`: The amount of elements to reference
-	///
-	/// Returns either _`Some(element_references)`_ if there are enough elements to reference or
-	/// _`None`_ otherwise
-	pub fn peek_n_mut(&mut self, n: usize) -> Option<&mut[T]> {
-		if self.len() < n { None }
-			else { Some(&mut self.backing[..n]) }
-	}
-	
-	
 	/// Appends `element` at the end
 	///
 	/// __Warning: This function panics if `self.limit` is exceeded__
@@ -296,11 +258,74 @@ impl<T> SliceQueue<T> {
 	/// Parameters:
 	///  - `src`: A slice containing the elements to clone and append
 	pub fn push_from(&mut self, src: &[T]) where T: Clone {
-		assert!(self.limit > self.len() + src.len(), "`self.len() + src.len()` is larger than `self.limit`");
+		assert!(self.limit >= self.len() + src.len(), "`self.len() + src.len()` is larger than `self.limit`");
 		
 		self.backing.extend_from_slice(src)
 	}
-	
+	/// Calls `push_fn` to push up to `n` elements in place
+	///
+	/// The function works like this:
+	///  1. `n` default elements are inserted at the end
+	///  2. `push_fn` is called with a mutable slice referencing the new elements and returns either
+	///     the amount of elements pushed or an error
+	///  3. If the amount of elements pushed is smaller than `n` or an error occurred, the unused
+	///     default elements are removed again
+	///
+	/// Example:
+	/// ```
+	/// # extern crate slice_queue;
+	/// # use slice_queue::SliceQueue;
+	///	let mut slice_queue = SliceQueue::new();
+	///
+	/// // Successful push
+	///	slice_queue.push_in_place(7, |buffer: &mut[usize]| -> Result<usize, ()> {
+	/// 	(0..4).for_each(|i| buffer[i] = i);
+	/// 	Ok(4)
+	/// });
+	/// assert_eq!(slice_queue.len(), 4);
+	/// (0..4).for_each(|i| assert_eq!(slice_queue[i], i));
+	///
+	/// // Failed push
+	/// slice_queue.push_in_place(7, |buffer: &mut[usize]| -> Result<usize, ()> {
+	/// 	(0..4).for_each(|i| buffer[i] = i + 7);
+	/// 	Err(())
+	/// });
+	/// assert_eq!(slice_queue.len(), 4);
+	/// (0..4).for_each(|i| assert_eq!(slice_queue[i], i));
+	///	```
+	pub fn push_in_place<E>(&mut self, n: usize, mut push_fn: impl FnMut(&mut[T]) -> Result<usize, E>) -> Result<(), E> where T: Default {
+		assert!(self.limit >= self.len() + n, "`self.len() + n` is larger than `self.limit`");
+		
+		// Append `n` default elements
+		let old_len = self.len();
+		#[cfg(feature = "unsafe_fast_code")]
+		unsafe {
+			// Reserve `n` elements and adjust length
+			self.backing.reserve(n);
+			self.backing.set_len(old_len + n);
+			
+			// Initialize the elements with their default value
+			let mut ptr = self.backing[old_len..].as_mut_ptr();
+			(0..n).for_each(|_| {
+				*ptr = T::default();
+				ptr = ptr.offset(1);
+			});
+		}
+		#[cfg(not(feature = "unsafe_fast_code"))]
+		/* safe */ {
+			(0..n).for_each(|_| self.backing.push(T::default()));
+		}
+		
+		// Call `push_fn` and truncate the length to the amount of elements pushed
+		let pushed = push_fn(&mut self.backing[old_len..]);
+		self.backing.truncate(old_len + match pushed.as_ref() {
+			Ok(pushed) => *pushed,
+			Err(_) => 0
+		});
+		self.shrink_opportunistic();
+		
+		pushed.map(|_| ())
+	}
 	
 	/// A private helper function to translate `RangeBounds` into ranges relative to `self`
 	///
